//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "FingerDlgs.h"
#include "HelpDlgs.h"
#include "FileMan.h"
#include "global.h"
//---------------------------------------------------------------------------
#pragma link "FingCli"
#pragma resource "*.dfm"
#define MaxFingerCount 16
#define BUFSIZE 16384
TFingerDlg *FingerDlg;
//---------------------------------------------------------------------------
void __fastcall RunFingerTool(TForm *Owner)
{
   FingerDlg = new TFingerDlg(Owner);
   CenterDialog(FingerDlg,Owner);
   FingerDlg->ShowModal();
   delete FingerDlg;
}
//---------------------------------------------------------------------------
__fastcall TFingerDlg::TFingerDlg(TComponent* Owner)
	: TForm(Owner)
{
   if (!AppDataExists(FingerHistoryKey))
   {
      TStringList *temps = new TStringList();
      GetAppData(temps,FingerHistoryKey,AppIni);
      if (temps->Count) PutAppData(FingerHistoryKey,temps);
      delete temps;
   }
   GetAppData(Query->Items,FingerHistoryKey);
   if (Query->Items->Count)
   {
      FingerBtn->Enabled = true;
      Query->ItemIndex = 0;
   }
   results = new TMemoryStream();
   lines = RichEdit1->Lines;
   RichEdit1->Clear();
   //Label1->Caption = "";
   editing = false;
   cap = Caption;
}
//---------------------------------------------------------------------------
__fastcall TFingerDlg::~TFingerDlg()
{
   delete results;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::CancelBtnClick(TObject *Sender)
{
   Timer1->Enabled = false;
   if (CancelBtn->Caption=="Cancel")
   {
      if (editing)
      {
         Query->Text = server;
         Query->Enabled = true;
         ActiveControl = Query;
         FingerBtn->Caption = "Finger";
         RichEdit1->ReadOnly = true;
         RichEdit1->Clear();
         Label1->Caption = "";
         editing = false;
         return;
      }
      FINGER->Abort();
      results->SetSize(0);
      RichEdit1->Clear();
      RichEdit1->Lines->Add("Query cancelled!");
      CancelBtn->Caption = "Close";
      Label1->Caption = "";
   }
   else ModalResult = mrCancel;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::QueryChange(TObject *Sender)
{
   FingerBtn->Enabled = !Query->Text.IsEmpty();
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FingerBtnClick(TObject *Sender)
{
   if (FingerBtn->Caption.Pos("Save")==1)
   {
      PutAppData(FingerHistoryKey,lines);
      Query->Items->Text = lines->Text;
      FingerBtn->Caption = "Finger";
      CancelBtn->Caption = "Close";
      Query->Enabled = true;
      Query->Text = server;
      ActiveControl = Query;
      Label1->Caption = "";
      RichEdit1->Clear();
      editing = false;
      return;
   }

   RichEdit1->Clear();
 	 AnsiString query = Query->Text;
   if (query.IsEmpty())
   {
      ShowMessage("No query string specified!");
      return;
   }

   TStrings *items = Query->Items;
   int idx = items->IndexOf(query);
   if (idx >= 0)
      items->Delete(idx);
   if (items->Count > 0)
      items->Insert(0,query);
   else
      items->Add(query);
   Query->ItemIndex = 0;

   if (query.Pos("@") == 0) query = "@"+query;
   FINGER->Query = query;
   ActiveControl = Panel1;
   results->SetSize(0);
   CancelBtn->Caption = "Cancel";
   timeout = 0; lines->Add("Connecting"); Timer1->Enabled = true;
   FingerBtn->Enabled = HelpBtn->Enabled = Query->Enabled = false;
   Label1->Caption = "";//Caption = cap;
   FINGER->StartQuery();
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FINGERSessionConnected(TObject *Sender, WORD Error)
{
   if (Error==0)
   {
      lines->Text = "Retrieving";
      timeout = 0;
   }
   else
   {
      Timer1->Enabled = false;
      FingerBtn->Enabled = HelpBtn->Enabled = Query->Enabled = true;
      CancelBtn->Caption = "Close";
   }
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FINGERDataAvailable(TObject *Sender, WORD Error)
{
   char *buf = new char[BUFSIZE];
   int len = FINGER->Receive(buf,BUFSIZE-1); buf[len] = '\0';
   results->Write(buf,len);
   delete buf;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FINGERQueryDone(TObject *Sender, WORD Error)
{
   SetRedraw(this,0);
   Timer1->Enabled = false;
 	 if (Error)
   {
      switch (Error)
      {
         case WSAECONNREFUSED: lines->Add("No finger service available."); break;
         case WSAETIMEDOUT: lines->Add("Host unreachable."); break;
         default:
            lines->Add("Error ="+String(Error));
      }
      FingerBtn->Enabled = HelpBtn->Enabled = Query->Enabled = true;
      CancelBtn->Caption = "Close";
      Label1->Caption = "";
      MyBeep(MB_OK);
   }
   else
   {
      Sleep(100);
      ActiveControl = RichEdit1;
      results->Seek(0,soFromBeginning);
      lines->LoadFromStream(results);
      CancelBtn->Caption = "Close";
      FingerBtn->Enabled = HelpBtn->Enabled = Query->Enabled = true;
      AppProcessMessages();
      MyBeep(MB_OK);

      AnsiString text = Query->Text;
      if (Query->ItemIndex>=0) Query->Items->Delete(Query->ItemIndex);
      Query->Items->Insert(0,text); Query->ItemIndex = 0;
      while (Query->Items->Count>MaxFingerCount)
         Query->Items->Delete(Query->Items->Count-1);
      PutAppData(FingerHistoryKey,Query->Items);
      Label1->Caption = s(RichEdit1->Lines->Count)+" items";
   }
   SetRedraw(this,1);
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::SaveBtnClick(TObject *Sender)
{
   TSaveDialog *sd = SaveDialog1;
   if (sd->InitialDir.IsEmpty())
       sd->InitialDir = ExtractFilePath(Application->ExeName);
   sd->FileName = "*.txt";
   sd->Filter = "Text files (*.txt)|*.TXT";
   sd->Options << ofHideReadOnly;
   if (sd->Execute())
      RichEdit1->Lines->SaveToFile(SaveDialog1->FileName);
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::Print1Click(TObject *Sender)
{
   RichEdit1->Print("Finger for "+Query->Text);
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::FormClose(TObject *Sender, TCloseAction &Action)
{
   Action = caFree;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::Timer1Timer(TObject *Sender)
{
   lines->Strings[0] = lines->Strings[0] + ".";
   if (timeout>=timeoutval*2)
   {
      Timer1->Enabled = false;
      FingerBtn->Enabled = false; HelpBtn->Enabled = Query->Enabled = false;
      FINGER->Abort();
      results->SetSize(0);
      RichEdit1->Clear();
      RichEdit1->Lines->Add("Query timed out!");
      CancelBtn->Caption = "Close";
      Label1->Caption = "";
      //Caption = cap;
      MyBeep(MB_OK);
   }
   timeout++;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::EditBtnClick(TObject *Sender)
{
   server = Query->Text;
   Query->Text = "Editing finger query list";
   CancelBtn->Caption = "Cancel";
   FingerBtn->Caption = "Save";
   GetAppData(lines,FingerHistoryKey);
   RichEdit1->ReadOnly = false;
   ActiveControl = RichEdit1;
   Query->Enabled = false;
   RichEdit1->SelStart = 0;
   editing = true;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::PopEditQueryListItemClick(TObject *Sender)
{
   EditBtnClick(NULL);
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::PopupMenu1Popup(TObject *Sender)
{
   PopEditQueryListItem->Enabled = !editing;
   PopCopyItem->Enabled = RichEdit1->SelLength>0;
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::PopCopyItemClick(TObject *Sender)
{
   RichEdit1->CopyToClipboard();
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::PopSelectAllItemClick(TObject *Sender)
{
   RichEdit1->SelectAll();
}
//---------------------------------------------------------------------------
void __fastcall TFingerDlg::HelpBtnClick(TObject *Sender)
{
   OpenHelp("Finger","Finger Client","",NULL);
}
//---------------------------------------------------------------------------
